require 'json'

module ExoLogging
  class ProviderFailure
    include Mongoid::Document
    include Mongoid::Timestamps

    field :name,     type: String
    field :provider, type: String

    validates_length_of :provider, minimum: 1

    @@app_name = 'Untitled'

    def self.set_app_name(appname)
      @@app_name = appname
    end

    def self.mk!(provider)
      x = ProviderFailure.new
      x.name     = @@app_name
      x.provider = provider

      x.save!
      x
    end

    def self.mtbf_helper(failed_ts, mtbf_type)
      failed_count = failed_ts.length

      last = -1
      mtbf = 0.0
      if failed_count > 0
        last = failed_ts[-1]
        tbf  = failed_ts[1..-1].zip(failed_ts).map { |pair| pair[0] - pair[1] }
        mtbf = nil
        case mtbf_type
        when 'mean'
          mtbf = ExoBasic::MeanAvg.new
        when 'short_term'
          mtbf = ExoBasic::ShortTermAvg.new
        else
          mtbf = ExoBasic::DecayingAvg.new
        end
        mtbf.offer_many(tbf)
      end

      {
        :count => failed_count,
        :last => last,
        :mtbf => mtbf.as_json
      }
    end

    def self.mtbf(provider, from=0, mtbf_type='decaying')
      from_time = ExoBasic::Timer.from_epoch(from).to_datetime
      failed_ts = ProviderFailure.where(name: @@app_name,
                                        provider: provider,
                                        :created_at.gte => from_time)
                                 .map { |i| ExoBasic::Timer.to_epoch(i.created_at) }
                                 .to_a
                                 .sort

      ProviderFailure.mtbf_helper(failed_ts, mtbf_type)
    end

  end
end
